from sqlalchemy import select
from sqlalchemy.orm import selectinload

from fastapi_user_auth.auth import Auth
from fastapi_user_auth.auth.models import User


async def test_create_role_user(auth: Auth):
    user = await auth.create_role_user("admin2")
    assert user.username == "admin2"
    # test user roles
    stmt = select(User).options(selectinload(User.roles)).where(User.username == "admin2")
    result = await auth.db.async_scalar(stmt)
    role = result.roles[0]
    assert role.key == "admin2"


async def test_authenticate_user(fake_auth: Auth):
    # error
    user = await fake_auth.authenticate_user("admin", "admin1")
    assert user is None

    # admin
    user = await fake_auth.authenticate_user("admin", "admin")
    assert user.username == "admin"



from sqlalchemy_database import AsyncDatabase, Database

# sqlite
sync_db = Database.create("sqlite:///amisadmin.db?check_same_thread=False")
async_db = AsyncDatabase.create("sqlite+aiosqlite:///amisadmin.db?check_same_thread=False")

# mysql
# sync_db = Database.create('mysql+pymysql://root:123456@127.0.0.1:3306/amisadmin?charset=utf8mb4')
# async_db = AsyncDatabase.create('mysql+aiomysql://root:123456@127.0.0.1:3306/amisadmin?charset=utf8mb4')

# postgresql
# sync_db = Database.create('postgresql://postgres:root@127.0.0.1:5432/amisadmin')
# async_db = AsyncDatabase.create('postgresql+asyncpg://postgres:root@127.0.0.1:5432/amisadmin')

# oracle
# sync_db = Database.create('oracle+cx_oracle://scott:tiger@tnsname')

# SQL Server
# sync_db = Database.create('mssql+pyodbc://scott:tiger@mydsn')



from datetime import date

import pytest
from fastapi import Depends
from fastapi_amis_admin.models.fields import Field

from fastapi_user_auth.auth.models import User


# 自定义`User`模型,继承`User`
class MyUser(User, table=True):
    point: float = Field(default=0, title="积分", description="用户积分")
    phone: str = Field(None, title="手机号", max_length=15)
    parent_id: int = Field(None, title="上级", foreign_key="auth_user.id")
    birthday: date = Field(None, title="出生日期")
    location: str = Field(None, title="位置")


@pytest.mark.parametrize("logins", ["admin"], indirect=True)
async def test_custom_user_model(fake_auth, logins):
    # 使用自定义的`User`模型,创建auth对象
    fake_auth.user_model = MyUser
    app = logins.app
    client = logins.client

    #  注册auth用户管理路由
    @app.get("/get_user")
    async def get_user(user: MyUser = Depends(fake_auth.get_current_user)):
        assert isinstance(user, MyUser)
        user.point = 100
        user.phone = "123456789"
        return user

    response = client.get("/get_user")
    data = response.json()
    assert "parent_id" in data
    assert data["point"] == 100
    assert data["phone"] == "123456789"



from typing import Tuple, Union

import pytest
from fastapi import Depends, HTTPException
from starlette.requests import Request
from starlette.responses import Response

from fastapi_user_auth.auth.auth import Auth
from fastapi_user_auth.auth.models import User
from tests.test_auth.conftest import UserClient


@pytest.fixture(autouse=True)
def setup(logins: UserClient):
    app = logins.app
    auth = logins.auth

    # auth decorator
    @app.get("/auth/user")
    @auth.requires()
    def user(request: Request):
        return request.user

    @app.get("/auth/admin_roles")
    @auth.requires("admin")
    def admin_roles(request: Request):
        return request.user

    @app.get("/auth/vip_roles")
    @auth.requires(["vip"])
    def vip_roles(request: Request):
        return request.user

    @app.get("/auth/admin_or_vip_roles")
    @auth.requires(roles=["admin", "vip"])
    def admin_or_vip_roles(request: Request):
        return request.user

    # auth async decorator
    @app.get("/auth/admin_roles_async")
    @auth.requires("admin")
    async def admin_roles_async(request: Request):
        return request.user

    # auth depend

    @app.get("/auth/user_1", dependencies=[Depends(auth.backend.authenticate)])
    def user_1(request: Request):
        if request.user:
            return request.user
        else:
            raise HTTPException(status_code=403)

    @app.get("/auth/user_2")
    def user_2(request: Request, auth_result: Tuple[Auth, User] = Depends(auth.backend.authenticate)):
        if request.user:
            return request.user
        else:
            raise HTTPException(status_code=403)

    @app.get("/auth/user_3")
    async def user_3(request: Request):
        if await auth.requires()(request):
            return request.user

    @app.get("/auth/user_4")
    async def user_4(request: Request, user: User = Depends(auth.get_current_user)):
        if user is None:
            raise HTTPException(status_code=403)
        return request.user

    @app.get("/auth/admin_roles_depend_1", dependencies=[Depends(auth.requires("admin")())])
    def admin_roles_1(request: Request):
        return request.user

    @app.get("/auth/admin_roles_depend_2")
    def admin_roles_2(request: Request, auth_result: Union[bool, Response] = Depends(auth.requires("admin")())):
        return request.user

    # auth group
    @app.get("/auth/admin_groups")
    @auth.requires(groups="admin")
    async def admin_groups(request: Request):
        return request.user

    @app.get("/auth/vip_groups")
    @auth.requires(groups=["vip"])
    async def vip_groups(request: Request):
        return request.user

    @app.get("/auth/admin_or_vip_groups")
    @auth.requires(groups=["admin", "vip"])
    async def admin_or_vip_groups(request: Request):
        return request.user

    # auth permission
    @app.get("/auth/permissions")
    @auth.requires(permissions=["test"])
    async def route(request: Request):
        return request.user


@pytest.mark.parametrize("logins", ["guest"], indirect=True)
def test_router_token(logins: UserClient):
    response = logins.client.post(
        "/auth/gettoken",
        data={"username": "admin", "password": "Incorrect"},
        headers={"Content-Type": "application/x-www-form-urlencoded"},
    )
    data = response.json()
    assert data["data"] is None


@pytest.mark.parametrize("logins", ["admin", "vip", "test", "guest"], indirect=True)
def test_router_userinfo(logins: UserClient):
    response = logins.client.get("/auth/userinfo")
    data = response.json()
    if logins.user:
        assert data["data"]["id"] == logins.user.id
        assert data["data"]["username"] == logins.user.username
    else:
        assert data["detail"] == "Forbidden"


path_all = {
    "/auth/user",
    "/auth/user_1",
    "/auth/user_2",
    "/auth/user_3",
    "/auth/user_4",
    # auth role
    "/auth/admin_roles",
    "/auth/vip_roles",
    "/auth/admin_or_vip_roles",
    # auth depend
    "/auth/admin_roles_depend_1",
    "/auth/admin_roles_depend_2",
    # auth group
    "/auth/admin_groups",
    "/auth/vip_groups",
    "/auth/admin_or_vip_groups",
    # auth test
    "/auth/permissions",
}
path_admin_auth = {
    "/auth/user",
    "/auth/user_1",
    "/auth/user_2",
    "/auth/user_3",
    "/auth/user_4",
    "/auth/admin_roles",
    "/auth/admin_or_vip_roles",
    "/auth/admin_roles_depend_1",
    "/auth/admin_roles_depend_2",
    "/auth/admin_roles_async",
    "/auth/admin_groups",
    "/auth/admin_or_vip_groups",
}

path_vip_auth = {
    "/auth/user",
    "/auth/user_1",
    "/auth/user_2",
    "/auth/user_3",
    "/auth/user_4",
    "/auth/vip_roles",
    "/auth/admin_or_vip_roles",
    "/auth/vip_groups",
    "/auth/admin_or_vip_groups",
}
path_test_auth = {
    "/auth/user",
    "/auth/user_1",
    "/auth/user_2",
    "/auth/user_3",
    "/auth/user_4",
    "/auth/permissions",
}


@pytest.mark.parametrize("logins", ["admin"], indirect=True)
@pytest.mark.parametrize("path", list(path_admin_auth))
def test_admin_auth(logins: UserClient, path):
    response = logins.client.get(path)
    data = response.json()
    assert data["id"] == logins.user.id
    assert data["username"] == logins.user.username


@pytest.mark.parametrize("logins", ["admin"], indirect=True)
@pytest.mark.parametrize("path", list(path_all - path_admin_auth))
def test_admin_forbidden(logins: UserClient, path):
    response = logins.client.get(path)
    data = response.json()
    assert data["detail"] == "Forbidden"


@pytest.mark.parametrize("logins", ["vip"], indirect=True)
@pytest.mark.parametrize("path", list(path_vip_auth))
def test_vip_auth(logins: UserClient, path):
    response = logins.client.get(path)
    data = response.json()
    assert data["id"] == logins.user.id
    assert data["username"] == logins.user.username


@pytest.mark.parametrize("logins", ["vip"], indirect=True)
@pytest.mark.parametrize("path", list(path_all - path_vip_auth))
def test_vip_forbidden(logins: UserClient, path):
    response = logins.client.get(path)
    data = response.json()
    assert data["detail"] == "Forbidden"


@pytest.mark.parametrize("logins", ["test"], indirect=True)
@pytest.mark.parametrize("path", list(path_test_auth))
def test_test_auth(logins: UserClient, path):
    response = logins.client.get(path)
    data = response.json()
    assert data["id"] == logins.user.id
    assert data["username"] == logins.user.username


@pytest.mark.parametrize("logins", ["guest"], indirect=True)
@pytest.mark.parametrize("path", list(path_all))
def test_guest_forbidden(logins: UserClient, path):
    response = logins.client.get(path)
    data = response.json()
    assert data["detail"] == "Forbidden"



import pytest

from fastapi_user_auth.auth.backends.db import DbTokenStore
from fastapi_user_auth.auth.backends.jwt import JwtTokenStore
from fastapi_user_auth.auth.schemas import BaseTokenData

token_data = BaseTokenData(id=1, username="test")


@pytest.mark.asyncio
async def test_jwt_token_store():
    store = JwtTokenStore(secret_key="09d25e094faa6ca2556c818166b7a9563b93f7099f6f0f4caa6cf63b88e8d3e7")
    token = await store.write_token(token_data)
    assert token
    data = await store.read_token(token=token)
    assert data == token_data
    with pytest.raises(NotImplementedError):
        await store.destroy_token(token)


@pytest.mark.asyncio
async def test_db_token_store(db):
    store = DbTokenStore(db)
    token = await store.write_token(token_data)
    assert token
    data = await store.read_token(token=token)
    assert data == token_data
    await store.destroy_token(token=token)
    data = await store.read_token(token=token)
    assert data is None


@pytest.mark.asyncio
async def test_redis_token_store():
    pass



import asyncio
from typing import Union

import pytest
from fastapi import FastAPI
from sqlalchemy.orm import Session
from sqlalchemy_database import AsyncDatabase, Database
from sqlmodel import SQLModel
from starlette.testclient import TestClient

from fastapi_user_auth.auth.auth import Auth, AuthRouter
from fastapi_user_auth.auth.models import Group, Permission, Role, User
from tests.conftest import async_db, sync_db


@pytest.fixture(params=[async_db, sync_db])
async def db(request) -> Union[Database, AsyncDatabase]:
    database = request.param
    await database.async_run_sync(SQLModel.metadata.create_all, is_session=False)
    yield database
    await database.async_run_sync(SQLModel.metadata.drop_all, is_session=False)


@pytest.fixture()
def auth(db: Union[Database, AsyncDatabase]) -> Auth:
    return Auth(db=async_db)


@pytest.fixture(scope="session")
def event_loop():
    loop = asyncio.get_event_loop_policy().new_event_loop()
    yield loop
    loop.close()


@pytest.fixture(scope="session")
async def fake_auth() -> Auth:
    auth = Auth(db=AsyncDatabase.create("sqlite+aiosqlite:///amisadmin.db?check_same_thread=False"))

    # noinspection PyTypeChecker
    def create_fake_users(session: Session):
        # init permission
        admin_perm = Permission(key="admin", name="admin permission")
        vip_perm = Permission(key="vip", name="vip permission")
        test_perm = Permission(key="test", name="test permission")
        session.add_all([admin_perm, vip_perm, test_perm])
        session.flush([admin_perm, vip_perm, test_perm])
        # init role
        admin_role = Role(key="admin", name="admin role", permissions=[admin_perm])
        vip_role = Role(key="vip", name="vip role", permissions=[vip_perm])
        test_role = Role(key="test", name="test role", permissions=[test_perm])
        session.add_all([admin_role, vip_role, test_role])
        session.flush([admin_role, vip_role, test_role])
        # init group
        admin_group = Group(key="admin", name="admin group", roles=[admin_role])
        vip_group = Group(key="vip", name="vip group", roles=[vip_role])
        test_group = Group(key="test", name="test group", roles=[test_role])
        session.add_all([admin_group, vip_group, test_group])
        session.flush([admin_group, vip_group, test_group])
        # init user
        admin_user = User(
            username="admin",
            password=auth.pwd_context.hash("admin"),
            email="admin@amis.work",
            roles=[admin_role],
            groups=[admin_group],
        )
        vip_user = User(
            username="vip", password=auth.pwd_context.hash("vip"), email="vip@amis.work", roles=[vip_role], groups=[vip_group]
        )
        test_user = User(
            username="test",
            password=auth.pwd_context.hash("test"),
            email="test@amis.work",
            roles=[test_role],
            groups=[test_group],
        )
        session.add_all([admin_user, vip_user, test_user])
        session.flush([admin_user, vip_user, test_user])

    await auth.db.async_run_sync(SQLModel.metadata.create_all, is_session=False)
    await auth.db.async_run_sync(create_fake_users)
    yield auth
    await auth.db.async_run_sync(SQLModel.metadata.drop_all, is_session=False)


class UserClient:
    def __init__(self, auth: Auth, client: TestClient = None, user: User = None) -> None:
        self.auth = auth
        self.app: FastAPI = client.app
        self.client: TestClient = client
        self.user: User = user


@pytest.fixture(scope="session")
def logins(request, fake_auth: Auth) -> UserClient:
    app = FastAPI()
    #  注册auth基础路由
    auth_router = AuthRouter(auth=fake_auth)
    app.include_router(auth_router.router)

    user_data = {
        "admin": {"username": "admin", "password": "admin"},
        "vip": {"username": "vip", "password": "vip"},
        "test": {"username": "test", "password": "test"},
        "guest": {"username": None, "password": None},
    }
    user = user_data.get(request.param) or {}

    def get_login_client(username: str = None, password: str = None) -> UserClient:
        client = TestClient(app)
        if not username or not password:
            return UserClient(fake_auth, client)
        response = client.post(
            "/auth/gettoken",
            data={"username": username, "password": password},
            headers={"Content-Type": "application/x-www-form-urlencoded"},
        )
        data = response.json()
        assert data["data"]["access_token"]
        user = User.parse_obj(data["data"])
        assert user.is_active
        assert user.username == username
        return UserClient(fake_auth, client=client, user=user)

    return get_login_client(**user)



import pytest
from fastapi import Depends, FastAPI, HTTPException
from starlette.requests import Request

from tests.test_auth.conftest import UserClient


@pytest.fixture(autouse=True)
def setup(logins: UserClient):
    app = logins.app
    auth = logins.auth
    subapp1 = FastAPI()
    app.mount("/subapp1", subapp1)

    subapp2 = FastAPI()
    app.mount("/subapp2", subapp2)
    auth.backend.attach_middleware(subapp2)

    subapp3 = FastAPI(dependencies=[Depends(auth.requires("admin")())])
    app.mount("/subapp3", subapp3)

    # auth decorator
    @subapp1.get("/auth/user")
    @auth.requires()
    def user(request: Request):
        return request.user

    @subapp2.get("/auth/user")
    def user_2(request: Request):
        if request.user:
            return request.user
        else:
            raise HTTPException(status_code=403)

    @subapp3.get("/auth/user")
    @auth.requires()
    def user_3(request: Request):
        return request.user


path_admin_auth = {
    "/subapp1/auth/user",
    "/subapp2/auth/user",
    "/subapp3/auth/user",
}


@pytest.mark.parametrize("logins", ["admin"], indirect=True)
@pytest.mark.parametrize("path", list(path_admin_auth))
def test_admin_auth(logins: UserClient, path):
    response = logins.client.get(path)
    data = response.json()
    assert data["id"] == logins.user.id
    assert data["username"] == logins.user.username
